/* ==========================================================================
 * ====                   FRACTAL GRAPHICS GENERATOR                     ====
 * ==========================================================================
 *
 * Copyright (C) 2003-2025 by Thomas Dreibholz
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contact: thomas.dreibholz@gmail.com
 */

#include "colorschemeinterface.h"

ClassRegistry* ColorSchemeInterface::Registry = nullptr;


// ###### Constructor #######################################################
ColorSchemeInterface::ColorSchemeInterface()
{
}


// ###### Destructor ########################################################
ColorSchemeInterface::~ColorSchemeInterface()
{
}


// ###### Configure color scheme ############################################
void ColorSchemeInterface::configure(const unsigned int* maxIterations)
{
   MaxIterations = maxIterations;
}


// ###### Create new instance class #########################################
bool ColorSchemeInterface::registerClass(const QString& identifier,
                                         const QString& description,
                                         ColorSchemeInterface* (*makeInstanceFunction)())
{
  if(Registry == nullptr) {
      Registry = new ClassRegistry;
      Q_CHECK_PTR(Registry);
  }
  return ColorSchemeInterface::Registry->registerClass(
            identifier, description,
            (void* (*)())makeInstanceFunction);
}
